@@ -3,7 +3,7 @@
 #
 #     This program is under GPL [http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt]
 #
-#     (c) 2007,2008,2009,2010 - Tom Eastep (teastep@shorewall.net)
+#     (c) 2007,2008,2009,2010,2011 - Tom Eastep (teastep@shorewall.net)
 #
 #     Traffic Control is from tc4shorewall Version 0.5
 #     (c) 2005 Arne Bernin <arne@ucbering.de>
@@ -40,9 +40,9 @@ use strict;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( setup_tc );
 our @EXPORT_OK = qw( process_tc_rule initialize );
-our $VERSION = '4.4_19';
+our $VERSION = '4.4_20';
 
-our %tcs = ( T => { chain  => 'tcpost',
+my  %tcs = ( T => { chain  => 'tcpost',
 		    connmark => 0,
 		    fw       => 1,
 		    fwi      => 0,
@@ -86,7 +86,7 @@ use constant { NOMARK    => 0 ,
 	       HIGHMARK  => 2
 	       };
 
-our %flow_keys = ( 'src'            => 1,
+my  %flow_keys = ( 'src'            => 1,
 		   'dst'            => 1,
 		   'proto'          => 1,
 		   'proto-src'      => 1,
@@ -104,14 +104,14 @@ our %flow_keys = ( 'src'            => 1,
 		   'sk-gid'         => 1,
 		   'vlan-tag'       => 1 );
 
-our %tosoptions = ( 'tos-minimize-delay'       => '0x10/0x10' ,
+my  %tosoptions = ( 'tos-minimize-delay'       => '0x10/0x10' ,
 		    'tos-maximize-throughput'  => '0x08/0x08' ,
 		    'tos-maximize-reliability' => '0x04/0x04' ,
 		    'tos-minimize-cost'        => '0x02/0x02' ,
 		    'tos-normal-service'       => '0x00/0x1e' );
-our %classids;
+my  %classids;
 
-our @deferred_rules;
+my  @deferred_rules;
 
 #
 # Perl version of Arn Bernin's 'tc4shorewall'.
@@ -132,43 +132,42 @@ our @deferred_rules;
 #                              name          => <interface>
 #                                               }
 #
-our @tcdevices;
-our %tcdevices;
-our @devnums;
-our $devnum;
-our $sticky;
-our $ipp2p;
+my  @tcdevices;
+my  %tcdevices;
+my  @devnums;
+my  $devnum;
+my  $sticky;
+my  $ipp2p;
 
 #
 # TCClasses Table
 #
-# %tcclasses { device    => <device> ,
-#              mark      => <mark> ,
-#              number    => <number> ,
-#              rate      => <rate> ,
-#              umax      => <umax> ,
-#              dmax      => <dmax> ,
-#              ceiling   => <ceiling> ,
-#              priority  => <priority> ,
-#              occurs    => <number> # 0 means that this is a class generated by another class with occurs > 1
-#              parent    => <class number>
-#              leaf      => 0|1
-#              guarantee => <sum of rates of sub-classes>
-#              options   => { tos  => [ <value1> , <value2> , ... ];
-#                             tcp_ack => 1 ,
-#                             ...
-#
+# %tcclasses { device    => <device> { number => { mark      => <mark> ,
+#                                                  rate      => <rate> ,
+#                                                  umax      => <umax> ,
+#                                                  dmax      => <dmax> ,
+#                                                  ceiling   => <ceiling> ,
+#                                                  priority  => <priority> ,
+#                                                  occurs    => <number> # 0 means that this is a class generated by another class with occurs > 1
+#                                                  parent    => <class number>
+#                                                  leaf      => 0|1
+#                                                  guarantee => <sum of rates of sub-classes>
+#                                                  options   => { tos  => [ <value1> , <value2> , ... ];
+#                                                  tcp_ack => 1 ,
+#                                                  ...
+#                                                }
+#                                     }
+#             }
+my  @tcclasses;
+my  %tcclasses;
 
-our @tcclasses;
-our %tcclasses;
-
-our %restrictions = ( tcpre      => PREROUTE_RESTRICT ,
+my  %restrictions = ( tcpre      => PREROUTE_RESTRICT ,
 		      tcpost     => POSTROUTE_RESTRICT ,
 		      tcfor      => NO_RESTRICT ,
 		      tcin       => INPUT_RESTRICT ,
 		      tcout      => OUTPUT_RESTRICT );
 
-our $family;
+my $family;
 
 #
 # Rather than initializing globals in an INIT block or during declaration,
@@ -332,18 +331,18 @@ sub process_tc_rule( ) {
 			    if ( defined $m1 && $m1 ne '' ) {
 				$val = numeric_value ($m1);
 				fatal_error "Invalid Mask ($m1)" unless defined $val && $val && $val <= 0xffffffff;
-				$mask1 = $m1;
+				$mask1 = in_hex ( $val & 0xffffffff );
 			    }
 
 			    if ( defined $m2 && $m2 ne '' ) {
 				$val = numeric_value ($m2);
 				fatal_error "Invalid Mask ($m2)" unless defined $val && $val <= 0xffffffff;
-				$mask2 = $m2;
+				$mask2 = in_hex ( $val & 0xffffffff );
 			    }
 
 			    if ( defined $s ) {
 				$val = numeric_value ($s);
-				fatal_error "Invalid Shift Bits ($s)" unless defined $val && $val < 128;
+				fatal_error "Invalid Shift Bits ($s)" unless defined $val && $val >= 0 && $val < 128;
 				$shift = $s;
 			    }
 			} else {
@@ -621,12 +620,15 @@ sub validate_tc_device( ) {
 	    $devnumber = hex_value( $number );
 	    fatal_error "Invalid device NUMBER ($number)" unless defined $devnumber && $devnumber && $devnumber < 256;
 	    fatal_error "Duplicate interface number ($number)" if defined $devnums[ $devnumber ];
-	    $devnum = $devnumber if $devnumber > $devnum;
 	} else {
 	    fatal_error "Missing interface NUMBER";
 	}
-    } elsif ( ( $devnumber = ++$devnum ) > 255 ) {
-	fatal_error "Attempting to assign a device number > 255";
+    } else {
+	1 while $devnums[++$devnum];
+
+	if ( ( $devnumber = $devnum ) > 255 ) {
+	    fatal_error "Attempting to assign a device number > 255";
+	}
     }
 
     $devnums[ $devnumber ] = $device;
@@ -1307,13 +1309,6 @@ sub process_tc_priority() {
 	return;
     }
 
-    fatal_error "Invalid tcpri entry" if ( $proto     eq '-' &&
-					   $ports     eq '-' &&
-					   $address   eq '-' &&
-					   $interface eq '-' &&
-					   $helper    eq '-' );
-
-
     my $val = numeric_value $band;
 
     fatal_error "Invalid PRIORITY ($band)" unless $val && $val <= 3;
@@ -1552,7 +1547,11 @@ sub setup_traffic_shaping() {
 
 	    if ( $tcref->{leaf} && ! $tcref->{pfifo} ) {
 		$sfqinhex = in_hexp( ++$sfq);
-		emit( "run_tc qdisc add dev $device parent $classid handle $sfqinhex: sfq quantum \$quantum limit $tcref->{limit} perturb 10" );
+		if ( $devref->{qdisc} eq 'htb' ) {
+		    emit( "run_tc qdisc add dev $device parent $classid handle $sfqinhex: sfq quantum \$quantum limit $tcref->{limit} perturb 10" );
+		} else {
+		    emit( "run_tc qdisc add dev $device parent $classid handle $sfqinhex: sfq limit $tcref->{limit} perturb 10" );
+		}
 	    }
 	    #
 	    # add filters
@@ -1605,8 +1604,11 @@ sub process_secmark_rule() {
 		 O => 'tcout'   , );
 
     my %state = ( N =>  'NEW' ,
+		  I => 'INVALID',
+		  NI => 'NEW,INVALID',
 		  E =>  'ESTABLISHED' ,
-		  ER => 'ESTABLISHED,RELATED' );
+		  ER => 'ESTABLISHED,RELATED',
+		);
 
     my ( $chain , $state, $rest) = split ':', $chainin , 3;
 
